from django.test import TestCase
from django.conf import settings

from django import forms


class MyForm(forms.Form):
    required_field = forms.CharField(required=True)
    optional_field = forms.CharField(required=False)


class FormTestCase(TestCase):

    def test_required_attr(self):
        """Only one field gets the required attribute"""
        form = MyForm()
        li = form.as_p().split("required=\"required\"")
        self.assertEqual(len(li), 2)

    def test_as_div(self):
        form = MyForm()
        self.failUnless("<div class=\"field\">" in form.as_div())


    def test_as_some_renderer(self):
        form = MyForm()
        self.failUnless("<div class=\"field\">" in form.as_some_renderer())



import inspect
import logging

from django.forms import BaseForm
from django.forms.widgets import Widget
from django.utils.module_loading import import_module
from django.conf import settings

from form_renderers import as_div


logger = logging.getLogger("logger")


# Add a required="required" attribute to widgets if applicable. This is a safe
# blanket policy.
def decorate(meth):
    def decorator(context, *args, **kwargs):
        di = meth(context, *args, **kwargs)
        if context.is_required and ("required" not in di):
            di["required"] = "required"
        return di
    return decorator


logger.info("Patching Widget.build_attrs")
Widget.build_attrs = decorate(Widget.build_attrs)


# Add the default as_div renderer
logger.info("Adding BaseForm.as_div")
BaseForm.as_div = as_div


# Add renderers from installed apps. Reverse the order so topmost apps can
# override other similarly named renderers.
for app_name in reversed(settings.INSTALLED_APPS):
    try:
        module = import_module(app_name + ".form_renderers")
    except ImportError:
        continue
    for name, func in inspect.getmembers(module, inspect.isfunction):
        logger.info("Adding BaseForm.%s" % name)
        setattr(BaseForm, name, func)



import os


DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
    }
}

INSTALLED_APPS = (
    "form_renderers",
    "form_renderers.tests",
    "django.contrib.contenttypes",
    "django.contrib.sites",
)

SITE_ID = 1

SECRET_KEY = "SECRET_KEY"



from setuptools import setup, find_packages


setup(
    name="django-form-renderers",
    version="0.1.1",
    description="Sometimes form.as_p doesn't cut it. This app adds more render methods to all forms.",
    long_description = open("README.rst", "r").read() + open("AUTHORS.rst", "r").read() + open("CHANGELOG.rst", "r").read(),
    author="Praekelt Consulting",
    author_email="dev@praekelt.com",
    license="BSD",
    url="http://www.jmbo.org",
    packages=find_packages(),
    install_requires=[
        "django",
    ],
    tests_require=[
        "tox",
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



def as_some_renderer(form):
    return form._html_output(
        normal_row=u"""<div class="field"><div %(html_class_attr)s>%(label)s %(errors)s <div class="helptext">%(help_text)s</div> %(field)s</div></div>""",
        error_row=u"%s",
        row_ender="</div>",
        help_text_html=u"%s",
        errors_on_separate_row=False
    )
